package de.jottyfan.timetrack.modules.done;

import static de.jottyfan.timetrack.db.done.Tables.T_OVERTIME;
import static de.jottyfan.timetrack.db.done.Tables.T_REQUIRED_WORKTIME;
import static de.jottyfan.timetrack.db.done.Tables.V_DAY;
import static de.jottyfan.timetrack.db.profile.Tables.T_LOGIN;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jooq.DSLContext;
import org.jooq.DatePart;
import org.jooq.Field;
import org.jooq.InsertOnDuplicateStep;
import org.jooq.Record1;
import org.jooq.Record3;
import org.jooq.Record5;
import org.jooq.SelectConditionStep;
import org.jooq.SelectHavingStep;
import org.jooq.UpdateConditionStep;
import org.jooq.impl.DSL;
import org.jooq.types.YearToSecond;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import de.jottyfan.timetrack.db.done.tables.records.TOvertimeRecord;
import de.jottyfan.timetrack.modules.done.model.DaysumBean;
import de.jottyfan.timetrack.modules.done.model.OvertimeBean;

/**
 * 
 * @author jotty
 * 
 */
@Repository
public class DoneRepository {
	private static final Logger LOGGER = LogManager.getLogger(DoneRepository.class);

	@Autowired
	private DSLContext jooq;

	public DaysumBean getDaysum(LocalDate day, String login) {
		Field<LocalTime> WORKTIME = DSL.field("worktime", LocalTime.class);
		Field<LocalTime> BREAKTIME = DSL.field("breaktime", LocalTime.class);
		
		SelectConditionStep<Record5<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond>> sql = jooq
		// @formatter:off
			.select(V_DAY.STARTTIME,
					    V_DAY.ENDTIME,
					    V_DAY.WORKTIME.cast(LocalTime.class).as(WORKTIME),
					    V_DAY.BREAKTIME.cast(LocalTime.class).as(BREAKTIME),
					    V_DAY.DAY_OVERTIME)
			.from(V_DAY)
			.innerJoin(T_LOGIN).on(T_LOGIN.PK.eq(V_DAY.FK_LOGIN))
			.where(V_DAY.DAY.eq(day))
			.and(T_LOGIN.LOGIN.eq(login));
		// @formatter:on
		LOGGER.trace(sql);
		Record5<LocalTime, LocalTime, LocalTime, LocalTime, YearToSecond> r = sql.fetchOne();
		if (r == null) {
			return null;
		} else {
			DaysumBean bean = new DaysumBean();
			bean.setDaytimeFrom(r.get(V_DAY.STARTTIME));
			bean.setDaytimeUntil(r.get(V_DAY.ENDTIME));
			bean.setDayworktime(r.get(WORKTIME));
			bean.setBreaks(r.get(BREAKTIME));
			YearToSecond dayOvertime = r.get(V_DAY.DAY_OVERTIME);
			Duration dayOvertimeDuration = dayOvertime == null ? null : dayOvertime.toDuration();
			bean.setDayOvertime(dayOvertimeDuration == null ? null : Long.valueOf(dayOvertimeDuration.toMinutes()).intValue());
			bean.setTotalOvertime(getOvertimeOf(day, login));
			return bean;
		}
	}
	
	private Integer getOvertimeOf(LocalDate day, String login) {
		Field<Integer> OVERTIME = DSL.field("overtime", Integer.class);
		SelectHavingStep<Record1<Integer>> sql = jooq
		// @formatter:off
			.select(T_OVERTIME.OVERTIME_MINUTES.plus(DSL.sum(DSL.extract(V_DAY.WORKTIME, DatePart.MINUTE).minus(T_REQUIRED_WORKTIME.REQUIRED_MINUTES))).as(OVERTIME))
			.from(V_DAY)
			.innerJoin(T_REQUIRED_WORKTIME).on(T_REQUIRED_WORKTIME.DAY.eq(V_DAY.DAY).and(T_REQUIRED_WORKTIME.FK_LOGIN.eq(V_DAY.FK_LOGIN)))
			.innerJoin(T_OVERTIME).on(T_OVERTIME.FK_LOGIN.eq(V_DAY.FK_LOGIN))
			.innerJoin(T_LOGIN).on(T_LOGIN.PK.eq(V_DAY.FK_LOGIN))
			.where(T_OVERTIME.IMPACT.ge(V_DAY.DAY.cast(LocalDateTime.class)))
			.and(V_DAY.DAY.le(day))
			.and(T_LOGIN.LOGIN.eq(login))
			.groupBy(V_DAY.FK_LOGIN, T_OVERTIME.OVERTIME_MINUTES);
		// @formatter:on
		LOGGER.trace(sql);
		return sql.fetchOne(OVERTIME);
	}

	public OvertimeBean getOvertimeBean(String login) {
		SelectConditionStep<Record3<Integer, LocalDateTime, Integer>> sql = jooq
		// @formatter:off
			.select(T_OVERTIME.PK_OVERTIME,
							T_OVERTIME.IMPACT,
							T_OVERTIME.OVERTIME_MINUTES)
			.from(T_OVERTIME)
			.innerJoin(T_LOGIN).on(T_LOGIN.PK.eq(T_OVERTIME.FK_LOGIN))
			.where(T_LOGIN.LOGIN.eq(login));
		// @formatter:on
		LOGGER.trace(sql);
		Record3<Integer, LocalDateTime, Integer> r = sql.fetchOne();
		OvertimeBean bean = new OvertimeBean();
		if (r == null) {
			bean.setImpact(LocalDate.now());
			bean.setOvertimeMinutes(0);
		} else {
			bean.setId(r.get(T_OVERTIME.PK_OVERTIME));
			bean.setImpact(r.get(T_OVERTIME.IMPACT).toLocalDate());
			bean.setOvertimeMinutes(r.get(T_OVERTIME.OVERTIME_MINUTES));
		}
		return bean;
	}

	public void upsertOvertime(Integer pkOvertime, String login, LocalDate impact, Integer overtimeMinutes) {
		if (pkOvertime == null) {
			InsertOnDuplicateStep<TOvertimeRecord> sql = jooq
			// @formatter:off
				.insertInto(T_OVERTIME,
						        T_OVERTIME.IMPACT,
						        T_OVERTIME.OVERTIME_MINUTES,
						        T_OVERTIME.FK_LOGIN)
				.select(jooq
					.select(DSL.val(impact == null ? null : impact.atStartOfDay()), DSL.val(overtimeMinutes), T_LOGIN.PK)
					.from(T_LOGIN)
					.where(T_LOGIN.LOGIN.eq(login)));
			// @formatter:on
			LOGGER.trace(sql);
			sql.execute();
		}
		UpdateConditionStep<TOvertimeRecord> sql = jooq
		// @formatter:off
			.update(T_OVERTIME)
			.set(T_OVERTIME.IMPACT, impact == null ? null : impact.atStartOfDay())
			.set(T_OVERTIME.OVERTIME_MINUTES, overtimeMinutes)
			.where(T_OVERTIME.PK_OVERTIME.eq(pkOvertime))
			.and(T_OVERTIME.FK_LOGIN.in(jooq
				.select(T_LOGIN.PK)
				.from(T_LOGIN)
				.where(T_LOGIN.LOGIN.eq(login))));
		// @formatter:on
		LOGGER.trace(sql);
		sql.execute();
	}
}
